const byte rev = 4;
// ALDL_BlueTooth_Gauge
//
// Mike Waldron 2013
// http://www.xm381.com/xm381/ALDL.html has ALDL info and documentation
//

// ***** $A1 ECU Configuration Info *********************************************************************************************
const byte Mode1Request[] = { 0xF4, 0x57, 0x01, 0x00 }; // Mode 1 Request byte string 
const byte Mode1Header[] = { 0xF4, 0x95, 0x01 };        // This is the header returned with the ALDL data
const byte Mode1DataBytes = 63;                         // This is the number of data bytes returned (not including the header)

// Status Bytes: ValueName = byte position in ALDLStream ( must offset location in ALDLStreamBuffer, offset = Mode1RequestSize + Mode1HeaderSize )  
const byte RPM = 11;  // *25 (ENGINE RPM)
const byte MAP = 29;  // *0.313 (MANIFOLD ABSOLUTE PRESSURE, kpa = (N + 28.06)/2.71)
const byte TPS = 10;  // *0.391 (RESCALED TPS, % THROTTLE = N/2.56)
const byte SPK = 38;  // (SPARK ADVANCE REL TO TDC (MSB) 2 byte, 2's complement data *0.352 [65536 - ((MSB * 256)+(LSB * 0.3515625))]
//const byte SPK1 = 40; // *0.352 - (SPARK ADVANCE, DEGREES = N*90/256 Values seem suspect, calc wrong?)
const byte AF = 41;   // *0.1 (DESIRED AIR/FUEL RATIO, A/F = N/10)
const byte WB = 31;   // *0.2 + 9.7 (wideband O2 connected to "fuel pump volt" input of ECU, volts = /10, A/F = volts/2 + 9, + .7 correction for my installation)
const byte BLM = 22;  // (BLOCK LEARN MULTIPLIER, 128 perfect, lower number leans too rich mixter, higher number enrichens too lean mixture)
const byte CTS = 7;   // (COOLANT TEMPERATURE, deg F = 1.35N - 40)
const byte MPH = 17;  // (FILTERED MILES PER HOUR)
const byte O2 = 19;   // (O2 mV = N * 4.42)

// More Status Bytes
//const byte BLC = 23;  // (CURRENT BLOCK LEARN MULTIPLIER CELL)
const byte BAT = 34;  // (BATTERY VOLTAGE, VOLTS = N/10)
//const byte IAC = 25;  // (IAC PRESENT MOTOR POSITION)
//const byte DIS = 27;  // (DESIRED IDLE SPEED, RPM = N*12.5)
//const byte PMP = 31;  // (FUEL PUMP RELAY, VOLTS = N/10)
//const byte AFG = 37;  // (ENGINE AIRFLOW, GRAMS/SECOND = N)
const byte BPW = 42;  // (FINAL BASE PULSE WIDTH (MSB), 2 byte, mSEC = (MSB*256 + LSB)/65.536)
//const byte RTM = 48;  // (ENGINE RUNNING TIME IN SECONDS (MSB), 2 byte, SECONDS = MSB*256 + LSB)
//const byte CNV = 50;  // (CATALYTIC CONVERTER TEMPERATURE (estimated by cpu, no sensor) DEGREES C = N*3 + 300)
const byte TBRP = 14; // (TIME BETWEEN REFERENCE PULSES (MSB), 2 byte, mSEC = (MSB*256 + LSB)/65.536)

// Status Bits: ValueName[] = {ALDL data byte, bit in data byte, true when bit is}
const byte Rich[] = {63,6};          // (1 = RICH 0 = LEAN) 
const byte OpenLoop[] = {63,7};      // (1 = IN CLOSED LOOP, 0 = OPEN LOOP)
const byte Idle[] = {55,7};          // (1 = IDLE CONDITIONS MET)
const byte ThrottleClosed[] = {62,7};// (1 = CLOSED TPS FLAG)
//const byte HighwayMode[] = {56,3};   // (1 = HIGHWAY SPARK ADVANCE (CONVERTER OVERTEMP MODE))
//const byte DFCO[] = {62,3};          // (DECEL FUEL CUTOFF FLAG)
const byte MALF[] = {58,1};          // (1 = MALFS HAVE OCCURRED)
const byte TCC[] = {52,3};           // (1 = TCC LOCKED / SHIFT LIGHT ON)

// More Status Bits
const byte FN1[] = {18,4};           //(1 = FAN #1 TO BE TURNED ON BY OUTPUT LOGIC)
//const byte FN2[] = {18,5};           //(1 = FAN #2 TO BE TURNED ON BY OUTPUT LOGIC)
//const byte ACC[] = {54,5};           //(0 = A/C CLUTCH ENABLED)
//const byte EGR[] = {56,4};           //(1 = EGR ON)
//const byte LMA[] = {58,2};           //(1 = LAUNCH MODE)
//const byte ACR[] = {59,7};           //(AIR CONDITIONER REQUEST, 1=ON   0=OFF)
//const byte IMF[] = {62,5};           //(IDLE MODE FUEL FLAG, 1 = CALCULATE BPW WITH AIRFLOW)
//const byte LRN[] = {63,1};           //(LEARN CONTROL ENABLE FLAG, 1=ENABLE)


// Malfunctions Status Bytes with "value" for each bit in an array
const byte MALFFLG1byte = 3;
const char* MALFFLG1 [] = { 
"VSS LOW",
"MAT LOW",
"TPS LOW",
"TPS HI",
"CTS LOW",
"CTS HIGH",
"O2 SENSOR",
"REF PULSE",
};
const byte MALFFLG2byte = 4;
const char* MALFFLG2[] = {
"EST ERR",
"CYLSEL ERR",
"IAC ERR",
"MAP LOW",
"MAP HIGH",
"EVRV/EGR",
"INJ MALF",
"MAT HI",
};
const byte MALFFLG3byte = 5;
const char* MALFFLG3[] = {
"ADU ERR",
"PUMP V LOW",
"BAT HIGH",
"",
"PROM ERR",
"O2 RICH",
"O2 LEAN",
"ESC ERR",
};
const byte MALFFLG4byte = 6;
const char* MALFFLG4[] = {
"",
"A/C SENSOR",
"",
"",
"TRANS SW",
"VATS ERR",
"DEGRADE O2",
"",
};

// ***** WBO2 Filter *****
float WBO2Filter; // smoothed WB data 
float O2Filter; // smoothed O2 data 

// MPG variables
/*  *** DutyCycle = BPW (BASE PULSE WIDTH) / TBRP (TIME BETWEEN REFERENCE PULSES) ***

ShortTermMPG: 
    every good sample [ShortTermMPG about 64 second sample period]
    increment byte MPGCounter
    populate GPHShortTerm[MPGCounter] with DutyCycle *** AvgDutyCycle = sum of GPHShortTerm[] / 256 ***
    populate MPHShortTerm[MPGCounter] with MPH *** AvgMPH = sum of MPHShortTerm[] / 256 ***
    AvgGPH = GPHInjector * AvgDutyCycle
    MPG = AvgMPH / AvgGPH
    BFUZ: Inj Fow Rate = 16.7 lb/hr   Display 2.668 gal/hr
    Jag:  Inj Fow Rate = 26.0 lb/hr   Display 4.42 gal/hr
    Jag has two 72 lb/hr - 12.25 gal/hr injectors
    Estimated HP = gal/hr * 15
    Injectors fire every third reference pulse 
*/
const float GPHInjector = 24.5; // 1 Pound/Hr = 0.17 Gallon/Hr
byte MPGCounter = 0;
int DutyCycle;
byte DutyCycleShortTerm[256];
byte MPHShortTerm[256];
int AvgMPH;
float AvgDutyCycle;
float MPG;


// ******************************************************************************************************************************


// ***** Hardware Config *****
//ALDL
#define ALDLSerial Serial

//HC05
#define Key 8 
#define RxD 7
#define TxD 6
#define State 5 //PIO11 - reset
#include <SoftwareSerial.h>
SoftwareSerial BluetoothSerial(RxD, TxD); // RX, TX

//Rotary Encoder
#include <EEPROM.h>

#define ENC_A 2
#define ENC_B 3
byte ENC_Count = 0;

byte Screens = 14; // number of screens (must match case: statement in ALDLcalc()
byte Screen; // current screen being displated

#include <Wire.h>

// ***** display Config Start *****
String ALDLvalue;
String ALDLlabel;
int sweep;

byte blk = 0x0;
byte wht = 0x1;
byte red = 0x2;
byte grn = 0x3;
byte yel = 0x4;
byte blu = 0x5;

byte sweepcolor;
byte indicator1;
byte indicator2;
byte indicator3;
byte indicator4;
// ***** display Config Stop *****

// ***** Variables for ALDL data *****
const byte Mode1RequestSize = sizeof(Mode1Request)/sizeof(Mode1Request[0]); // Calculate how many bytes long is the Mode 1 request string.
const byte Mode1HeaderSize = sizeof(Mode1Header)/sizeof(Mode1Header[0]); // Calculate how many bytes long is the Mode 1 header string.
const byte ALDLStreamSize = (Mode1RequestSize + 1 + Mode1HeaderSize + Mode1DataBytes + 1);  // # of Request bytes + checksum byte + header bytes + data bytes + checksum byte
byte offset = Mode1RequestSize + Mode1HeaderSize; //starting location of ALDL in buffer (after request and header data)
byte ALDLStream[ALDLStreamSize]; // Array for verified ALDL data
byte ALDLStreamBuffer[ALDLStreamSize]; // Array to store ALDL for verification 
byte checkSum;
byte Mode1RequestcheckSumByte; // Checksum of Mode 1 bytes
byte ALDLValidity;
unsigned long timeout;

// ***** Program Setup *********************************************************************************
void setup() {
  ALDLSerial.begin(8192); 
  BluetoothSerial.begin(9600);

  Wire.begin(); // join i2c bus (address optional for master)
  
// ***** setup BlueTooth *****
   pinMode(State, INPUT);
   pinMode(Key, INPUT);

// ***** setup encoder *****
  Screen = EEPROM.read(0);

  pinMode(ENC_A, INPUT);
  digitalWrite(ENC_A, HIGH);
  pinMode(ENC_B, INPUT);
  digitalWrite(ENC_B, HIGH);

  attachInterrupt(0, Encoder, CHANGE);
  attachInterrupt(1, Encoder, CHANGE);

// ***** Mode 1 checksum calculation *****
  checkSum = 0;
  for (byte Requestbytes=0; Requestbytes < Mode1RequestSize - 1; Requestbytes++) {
    checkSum += Mode1Request[Requestbytes];
  }
  Mode1RequestcheckSumByte = 0 - checkSum;
  
}

// ***** Main Loop *********************************************************************************
void loop() {
  
  if (digitalRead(State) == HIGH){  
  //paired
    BluetoothRequest();
  }
    else{  
    //not paired  if (digitalRead(State) == HIGH){  
      beginMode1(); // request mode 1
    
    // get ALDL data
      getALDLStream(); 

    // Verify ALDLStream    
      VerifyALDLStream();
  
    // Calculate Values
      ALDLcalc();
    
    // check rotary encoder
//      Encoder();

      MPGCalc();
  }
  
  //
    transmit();  
}

// ***** Bluetooth Request *********************************************************************************
void BluetoothRequest() {

  if (BluetoothSerial.available()) { // retrieve ALDL request data from PC in buffer, if any
    while(BluetoothSerial.available()){ // empty serial buffer
      ALDLSerial.write(BluetoothSerial.read());
    }
  }
  
  if (ALDLSerial.available()) { // retrieve ALDL response data from ECU in buffer, if any
    while(ALDLSerial.available()){ // empty serial buffer
      BluetoothSerial.write (ALDLSerial.read());
    }
  }

  ALDLvalue="BT";
  ALDLlabel="8192";    
  
}

// ***** Mode 1 Request *********************************************************************************
void beginMode1() {
   
  while(ALDLSerial.available() != 0){ // empty serial buffer
    byte bitbucket = ALDLSerial.read();
  }
  
  for(byte bytesRead = 0; bytesRead <= Mode1RequestSize - 1; bytesRead++) { // send request  
    ALDLSerial.write (Mode1Request[bytesRead]);
  }
  ALDLSerial.write (Mode1RequestcheckSumByte); // send checksum
}

// ***** getALDLStream *****
void getALDLStream() {
  
  for (byte Clearbytes=0; Clearbytes < ALDLStreamSize; Clearbytes++) { // empty ALDL Stream array
    ALDLStream[Clearbytes] = 0x00;
  }
        
  for (byte Clearbytes=0; Clearbytes < ALDLStreamSize; Clearbytes++) { // empty ALDL Stream Buffer array
    ALDLStreamBuffer[Clearbytes] = 0x00;
  }
  
  byte bytesRead = 0;      
  timeout = millis();
  
  while((millis() - timeout) < 100){
    if (ALDLSerial.available()) { // retrieve ALDL response data from ECU in buffer, if any
      byte dataByte = ALDLSerial.read();
      ALDLStreamBuffer[bytesRead] = dataByte;
//      BluetoothSerial.write (dataByte);
      bytesRead++;
    }
    if(bytesRead >= ALDLStreamSize){
      break;
    } 
  }
}

// verify ALDL data stream: Mode1Request + checksum + header + ALDL data + checksum byte should equal 0               
// ***** Verify ALDLStream *********************************************************************************
void VerifyALDLStream(){
  // check header string in ALDL data stream: does it match Mode1Header string        
  byte Mode1HeadercheckSum=0;
       checkSum=0;

  for (byte Requestbytes=0; Requestbytes < Mode1HeaderSize; Requestbytes++) {
    Mode1HeadercheckSum += Mode1Header[Requestbytes];
    checkSum += ALDLStreamBuffer[Requestbytes + Mode1RequestSize + 1];
  }

  if (Mode1HeadercheckSum != checkSum) {
    ALDLValidity = 2;
    return;
  }

  // check ALDL data stream: header + ALDL data + checksum byte should equal 0               
  checkSum=0;
  
  for (byte Requestbytes=0; Requestbytes < ALDLStreamSize - (Mode1RequestSize + 1); Requestbytes++) {
    checkSum += ALDLStreamBuffer[Requestbytes + (Mode1RequestSize + 1)];
  }

  if (checkSum != 0) {
    ALDLValidity = 1;
    return;
  }

  if (checkSum == 0) {
  ALDLValidity = 0;
  
    for(byte bytesRead = 0; bytesRead < ALDLStreamSize ; bytesRead++){
      ALDLStream[bytesRead] = ALDLStreamBuffer[bytesRead]; // transfer good data to ALDLStream     
      BluetoothSerial.write ( ALDLStreamBuffer[bytesRead]);
    }
  } 
}


// ***** ALDLCalc Values *********************************************************************************
void ALDLcalc(){
  
  char buffer[5];
  float temp;
  
  switch(Screen){
    
    case 0:       //MPH min-0, max 120
      ALDLlabel = "MPH";
      temp = ALDLStream[MPH + offset];
      ALDLvalue = dtostrf(temp, 3, 0, buffer);
      sweep = constrain((temp / 10) + 128, 128, 139) ;
      if (temp > 75){
        sweepcolor = red;
      }
      else if (temp < 60){
        sweepcolor = grn;
      }
      else{
        sweepcolor = yel;
      }
      break;
    
    case 1:       //RPM min-500, max 5500
      ALDLlabel = "RPM";                                    // text to be displayed on second line
      temp = ALDLStream[RPM + offset] * 25;                 // calculate value
      ALDLvalue = dtostrf(temp, 4, 0, buffer);              // convert calculated value to text
      sweep = constrain(((temp - 500) /  458) + 128, 128, 139);  // caclulate sweep endpoint, adding 128 displays continous sweep otherwise just a dot 
      if (temp > 4500){                                     // set sweep color
        sweepcolor = red;
      }
      else if (temp < 3000){
        sweepcolor = grn;
      }
      else{
        sweepcolor = yel;
      }
      break;
    
    case 2:       //MAP min-0, max 99
      ALDLlabel = "MAP";
      temp = ALDLStream[MAP + offset] * 0.313;
      ALDLvalue = dtostrf(temp, 2, 0, buffer);
      sweep = constrain((temp / 8.25) + 128, 128, 139) ;
      sweepcolor = grn;
      break;
    
    case 3:       //TPS min-0, max 99
      ALDLlabel = "TPS";
      temp = ALDLStream[TPS + offset] * 0.391;
      ALDLvalue = dtostrf(temp, 2, 0, buffer);
      sweep = constrain((temp / 8.25) + 128, 128, 139) ;
      if (bitRead (ALDLStream[Idle[0] + offset], Idle[1]) == 1){ // (1 = IDLE CONDITIONS MET)
        sweepcolor = blu;
      }
      else{
        sweepcolor = grn;
      }
     break;
    
    case 4:       //O2 min-0, max 999
      ALDLlabel = "O2";
//      temp = ALDLStream[O2 + offset] * 4.42;
      temp = O2Calc();
      ALDLvalue = dtostrf(temp, 3, 0, buffer);
      sweep = constrain(temp / 82.5, 0, 11) ;
      if (temp < 500){
        sweepcolor = blu;
      }
      else{
        sweepcolor = red;
      }
      break;
    
    case 5:       //A/F min-9, max-17
      ALDLlabel = "A/F";
      temp = ALDLStream[AF + offset] * 0.1;
      ALDLvalue = dtostrf(temp, 4, 1, buffer);
      sweep = constrain((temp - 9) / 0.66, 0, 11) ;
      if (temp > 15.2){
        sweepcolor = blu;
      }
      else if (temp < 14.2){
        sweepcolor = red;
      }
      else{
        sweepcolor = grn;
      }
     break;
    
    case 6:       //WBO2 min-9, max-17
      ALDLlabel = "WBO2";
//      temp = ALDLStream[WB + offset] * 0.2 + 9.7;
      temp = WBO2Calc();
      if(temp == 9.7){
        ALDLvalue = "==";
      }
      else{
        ALDLvalue = dtostrf(temp, 4, 1, buffer);
      }
      sweep = constrain((temp - 9) / 0.66, 0, 11) ;
      if (temp > 15.2){
        sweepcolor = blu;
      }
      else if (temp < 14.2){
        sweepcolor = red;
      }
      else{
        sweepcolor = grn;
      }
      break;
    
    case 7:       //BLM min-104, max-152
      ALDLlabel = "BLM";
      temp = ALDLStream[BLM + offset];
      ALDLvalue = dtostrf(temp, 3, 0, buffer);
      sweep = constrain((temp - 104) / 4.36, 0, 11) ;
      if (temp > 136){
        sweepcolor = blu;
      }
      else if (temp < 120){
        sweepcolor = red;
      }
      else{
        sweepcolor = grn;
      }
      break;

    case 8:       //SPK min-0, max 48
      ALDLlabel = "SPK";
      if(ALDLStream[SPK + offset] >= 128){
       temp = 65536 - (((ALDLStream[SPK + offset] * 256) + ALDLStream[SPK + 1 + offset]) * 0.3515625);
       ALDLvalue = dtostrf(temp, 3, 0, buffer);
       sweep = constrain(temp / 4, 0, 11) ;
      }
      else{
       temp = ((ALDLStream[SPK + offset] * 256) + ALDLStream[SPK + 1 + offset]) * 0.351562;
       ALDLvalue = dtostrf(temp, 3, 0, buffer);
       sweep = constrain(temp / 4, 0, 11) ;
      }
      sweepcolor = grn;
      break;
    
    case 9:       //CTS min-120, max 240
      ALDLlabel = "CTS";
      temp = (ALDLStream[CTS + offset] * 1.35) - 40;
      ALDLvalue = dtostrf(temp, 3, 0, buffer);
      sweep = constrain(((temp - 120) / 10) + 128, 128, 139);
      if (temp > 200){
        sweepcolor = red;
      }
      else if (temp < 150){
        sweepcolor = blu;
      }
      else{
        sweepcolor = grn;
      }
    break;
    
    case 10:       //VOLTS min-10, max-17
      ALDLlabel = "VOLTS";
      temp = ALDLStream[BAT + offset] * 0.1;
      ALDLvalue = dtostrf(temp, 4, 1, buffer);
      sweep = constrain(((temp - 10) / 0.58) + 128, 128, 139) ;
      if (temp > 14.5){
        sweepcolor = red;
      }
      else if (temp < 13){
        sweepcolor = red;
      }
      else{
        sweepcolor = grn;
      }
      break;

    case 11:       //MPG min-0, max-30
      ALDLlabel = "MPG";
      temp = MPG;
      ALDLvalue = dtostrf(temp, 3, 0, buffer);
      sweep = constrain(temp / 3, 0, 11) ;
      if (temp > 24){
        sweepcolor = blu;
      }
      else if (temp < 15){
        sweepcolor = red;
      }
      else{
        sweepcolor = grn;
      }
      break;

    case 12:       //freeRam
      ALDLlabel = "freeRam";
      temp = freeRam();
      ALDLvalue = dtostrf(temp, 3, 0, buffer);
      sweep = 0;
      sweepcolor = blk;
      break;

    case 13:      // MALFS
      if (ALDLStream[MALFFLG1byte + offset] > 0 || ALDLStream[MALFFLG2byte + offset] > 0 || ALDLStream[MALFFLG3byte + offset] > 0 || ALDLStream[MALFFLG4byte + offset] > 0){

      ALDLvalue = "MALF";

      for(byte MALFbit = 0; MALFbit < 8 ; MALFbit++){
            if(bitRead (ALDLStream[MALFFLG4byte + offset], MALFbit) == 1){ALDLlabel = MALFFLG4[MALFbit];}
      }
      for(byte MALFbit = 0; MALFbit < 8 ; MALFbit++){
            if(bitRead (ALDLStream[MALFFLG3byte + offset], MALFbit) == 1){ALDLlabel = MALFFLG3[MALFbit];}
      }
      for(byte MALFbit = 0; MALFbit < 8 ; MALFbit++){
            if(bitRead (ALDLStream[MALFFLG2byte + offset], MALFbit) == 1){ALDLlabel = MALFFLG2[MALFbit];}
      }
      for(byte MALFbit = 0; MALFbit < 8 ; MALFbit++){
            if(bitRead (ALDLStream[MALFFLG1byte + offset], MALFbit) == 1){ALDLlabel = MALFFLG1[MALFbit];}
      }
    }
    else{
        ALDLvalue = "NO";
        ALDLlabel = "MALFs";
    }
      sweep = 0;
      sweepcolor = blk;
    break;
    
    case 14:      // ALDL Verify
    ALDLvalue = "ALDL";

    if (ALDLValidity == 0){
       ALDLlabel = "good";
      }
      else{
       ALDLlabel = "fault";
      }
      sweep = 0;
      sweepcolor = blk;
    break;
    
  }

  if(bitRead (ALDLStream[FN1[0] + offset], FN1[1]) == 1){ //(1 = FAN #1 TO BE TURNED ON BY OUTPUT LOGIC)
    indicator1 = red;
  }
  else{
    indicator1 = blk;
  } 
  
  if(bitRead (ALDLStream[OpenLoop[0] + offset], OpenLoop[1]) == 0){ // (1 = IN CLOSED LOOP, 0 = OPEN LOOP)
    indicator2 = grn;
  }
  else if(bitRead (ALDLStream[Rich[0] + offset], Rich[1]) == 1){ // (1 = RICH 0 = LEAN)
    indicator2 = red;
  }
  else {
    indicator2 = blu;
  }

  if(bitRead (ALDLStream[TCC[0] + offset], TCC[1]) == 1){ // (1 = TCC LOCKED / SHIFT LIGHT ON)
    indicator3 = blu;
  }
  else if (bitRead (ALDLStream[Idle[0] + offset], Idle[1]) == 1){ // (1 = IDLE CONDITIONS MET)
    indicator3 = grn;
  }
  else{
    indicator3 = blk;    
  }


  if(bitRead (ALDLStream[MALF[0] + offset], MALF[1]) == 1){ // (1 = MALFS HAVE OCCURRED)
    indicator4 = red;
  }
  else{
    indicator4 = blk;
  }
}


// ************************************************** MPG **************************************************
int WBO2Calc(){

    WBO2Filter = WBO2Filter - (WBO2Filter/3.0) + ((ALDLStream[WB + offset] * 0.2) + 9); // smooth WBO2 data    

    return WBO2Filter;
}

// ************************************************** MPG **************************************************
int O2Calc(){

    O2Filter = O2Filter - (O2Filter/3.0) + (ALDLStream[O2 + offset] * 4.42); // smooth WBO2 data    

    return O2Filter;
}

// ************************************************** MPG **************************************************
void MPGCalc(){

 if (ALDLValidity == 0){

    MPGCounter += 1;
    
    DutyCycleShortTerm[MPGCounter] =  100 * ((float)((ALDLStream[BPW + offset] * 256) + ALDLStream[BPW + offset + 1] ) / (((ALDLStream[TBRP + offset] * 256) + ALDLStream[TBRP + offset + 1] ) * 3));
    MPHShortTerm[MPGCounter] = ALDLStream[MPH + offset];

    DutyCycle = 0;        
    for (byte Requestbytes=0; Requestbytes != 255; Requestbytes++) {
       DutyCycle += DutyCycleShortTerm[Requestbytes];
    }
    AvgDutyCycle = DutyCycle / 25500.0;
    
    AvgMPH = 0;
    for (byte Requestbytes=0; Requestbytes != 255; Requestbytes++) {
       AvgMPH += MPHShortTerm[Requestbytes];
    }
    AvgMPH = AvgMPH / 255;
    
    MPG = (AvgMPH * 1.0) / (AvgDutyCycle * GPHInjector);    

 }
}

// ***** Calc Free Ram *********************************************************************************
int freeRam () {
  extern int __heap_start, *__brkval; 
  int v; 
  return (int) &v - (__brkval == 0 ? (int) &__heap_start : (int) __brkval); 
}

// ***** Rotary Encoder *********************************************************************************
void Encoder()
{
  static int8_t Enc_States[] = {0,-1,1,0,1,0,0,-1,-1,0,0,1,0,1,-1,0};
  static uint8_t Enc_AB = 0;
  /**/
  Enc_AB <<= 2;                   //remember previous state
  bitWrite(Enc_AB, 0, digitalRead(ENC_A));
  bitWrite(Enc_AB, 1, digitalRead(ENC_B));

  ENC_Count += Enc_States[( Enc_AB & 0x0f )];

  if (ENC_Count == 255){
    ENC_Count = Screens * 4;
  }
  else if (ENC_Count / 4 > Screens){
    ENC_Count = 0;
  }
  
  Screen = ENC_Count / 4;

  EEPROM.write(0, Screen);    // Store screen number to return to same screen after power cycle

}
// ***** transmit *********************************************************************************
void transmit(){
  Wire.beginTransmission(4); // transmit to device #4
  Wire.write(sweep);       //1 byte
  Wire.write(sweepcolor);  //1 byte
  Wire.write(indicator1);  //1 byte
  Wire.write(indicator2);  //1 byte
  Wire.write(indicator3);  //1 byte
  Wire.write(indicator4);  //1 byte  
  
  if (digitalRead(State) == HIGH){  
  //paired
    Wire.write(0x1);       //1 byte
  }
  else{  
   Wire.write(0x0);        //1 byte 
  }

  Wire.write(ALDLValidity);//1 byte  
  Wire.print(ALDLvalue);   //4 bytes
  Wire.print(",");         //1 byte
  Wire.print(ALDLlabel);   //10 bytes

  Wire.write('\0');
  Wire.endTransmission();    // stop transmitting
}




