const byte rev = 4;
// ALDL_BlueTooth_Gauge
//
// Mike Waldron 2013
// http://www.xm381.com/xm381/ALDL.html has ALDL info and documentation
//

// ***** Hardware Config *****
#define DebugSerial Serial

// ***** Display Config Start ***** 
#include <Adafruit_NeoPixel.h>
// Parameter 1 = number of pixels in strip
// Parameter 2 = pin number (most are valid)
// Parameter 3 = pixel type flags, add together as needed:
//   NEO_RGB     Pixels are wired for RGB bitstream
//   NEO_GRB     Pixels are wired for GRB bitstream
//   NEO_KHZ400  400 KHz bitstream (e.g. FLORA pixels)
//   NEO_KHZ800  800 KHz bitstream (e.g. High Density LED strip)
Adafruit_NeoPixel strip = Adafruit_NeoPixel(16, 3, NEO_GRB + NEO_KHZ800);

#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

#define OLED_CS 4
#define OLED_RESET 5
#define OLED_DC 6
#define OLED_CLK 7
#define OLED_MOSI 8
Adafruit_SSD1306 display(OLED_MOSI, OLED_CLK, OLED_DC, OLED_RESET, OLED_CS);

char inChar;

byte sweep;
byte sweepcolor;

byte indicator1;
byte indicator2;
byte indicator3;
byte indicator4;

String ALDLvalue;
String ALDLlabel;

boolean stringComplete=false;
int index=0;
byte DisplayData[25];

//                     black     white     red       green     yellow    blue      
const long colors[] = {0x000000, 0x0f0f0f, 0x0f0000, 0x000f00, 0x0e0e01, 0x00000f};

// ***** Program Setup *********************************************************************************
void setup() {

  Wire.begin(4);                // join i2c bus with address #4
  Wire.onReceive(receiveEvent); // register event
    
// ***** Display Config Start *****
  // by default, we'll generate the high voltage from the 3.3v line internally! (neat!)
  display.begin(SSD1306_SWITCHCAPVCC);
  // init done
  display.clearDisplay();   // clears the screen and buffer
  display.display();

  strip.begin();
  strip.show(); // Initialize all pixels to 'off'

  display.clearDisplay();   // clears the screen and buffer
  display.setTextColor(WHITE);

  display.setCursor(8,0);
  display.setTextSize(5);
  display.print("ALDL");

  display.setCursor(32,38);
  display.setTextSize(2);
  display.print("gauge");
  
  display.setCursor(0,56);
  display.setTextSize(1);
  display.print("Version:");
  display.print(rev);

  display.setCursor(64,56);
  display.setTextSize(1);
  display.print("ram:");
  display.print(freeRam());
  display.display();
 
//  delay(2000);
  
  // Some example procedures showing how to display to the pixels:
  colorWipe(strip.Color(4, 0, 0), 50); // Red
  colorWipe(strip.Color(0, 4, 0), 50); // Green
  colorWipe(strip.Color(0, 0, 4), 50); // Blue
// ***** Display Config Stop *****

}

// ***** Main Loop *********************************************************************************
void loop() {
   
  DebugSerial.print("RAM: "); 
  DebugSerial.println(freeRam());
    
  // Display on Gauge
    ALDLdisplay();
}


// **** Fill the dots one after the other with a color *********************************************************************************
void colorWipe(uint32_t c, uint8_t wait) {
  for(uint16_t i=0; i<strip.numPixels(); i++) {
      strip.setPixelColor(i, c);
      strip.show();
      delay(wait);
  }
}

// ***** ALDLDisplay *********************************************************************************
void ALDLdisplay(){
  
  display.clearDisplay();   // clears the screen and buffer
  display.setTextColor(WHITE);

  display.setTextSize(5);
  display.setCursor(64 - (ALDLvalue.length() * 16),0);
  display.print(ALDLvalue);

  display.setTextSize(2);
  display.setCursor(64 - (ALDLlabel.length() * 6),48);
  display.print(ALDLlabel);

  if(DisplayData[7] != 0){ // if == 0 ALDL data good
    display.setTextSize(2);
    display.setCursor(0,48);
    display.print("!");
  }

  if(DisplayData[6] == 1){  //if bluetooth paired show BT symbol
    display.setTextSize(1);
    display.setCursor(118,48);
    display.print("|");
    display.setCursor(118,56);
    display.print("|");  
    display.setCursor(115,52);
    display.print(">");
    display.setCursor(120,48);
    display.print(">");
    display.setCursor(120,56);
    display.print(">");  
  }
  
  display.display();

  //clear pixels  
  for(byte i=0; i<strip.numPixels(); i++) {
    strip.setPixelColor(i, 0);
  }

  if(DisplayData[6] != 1){  //if not paired show pixels
    
    if(DisplayData[0] >= 128) { // diplay continuous swep
      for(byte i=0; i < (DisplayData[0] - 127); i++) {
        strip.setPixelColor(15 - i, colors[DisplayData[1]]);
      }
    }
    else { // display dot
      strip.setPixelColor(15 - DisplayData[0], colors[DisplayData[1]]);
    }
    
    strip.setPixelColor(3, colors[DisplayData[2]]);
    strip.setPixelColor(2, colors[DisplayData[3]]);
    strip.setPixelColor(1, colors[DisplayData[4]]);
    strip.setPixelColor(0, colors[DisplayData[5]]);
  }
  
  strip.show();  
}

// ***** Calc Free Ram *********************************************************************************
int freeRam () {
  extern int __heap_start, *__brkval; 
  int v; 
  return (int) &v - (__brkval == 0 ? (int) &__heap_start : (int) __brkval); 
}

// ***** I2C Wire Receive *********************************************************************************
// function that executes whenever data is received from master
// this function is registered as an event, see setup()
void receiveEvent(int howMany){

  index = 0;
  // Read while we have data
 while (Wire.available()){
   DisplayData[index] = Wire.read();
   // Increment where to write next
   index++;
  }
  
  index = 8;
  
  ALDLvalue = "";
  while (DisplayData[index] != ',' && index < 25){
    ALDLvalue += char (DisplayData[index]);    
    index++;
  }
  
  index++;
  ALDLlabel = "";
  while (DisplayData[index] != '\0' && index < 25){
    ALDLlabel += char (DisplayData[index]);    
    index++;
  }
}






